require 'kubeclient'
require 'celluloid/io'
# require 'singleton'

module DTK
  class CrdClient
    # include Singleton
    DEFAULT_API_VERSION = 'v1alpha1'

    # COMPONENT_DEF_CRD_VERSION     = ENV["COMPONENT_DEF_CRD_VERSION"]
    # ASSEMBLY_CRD_VERSION          = ENV["ASSEMBLY_CRD_VERSION"]
    # WORKFLOW_CRD_VERSION          = ENV["WORKFLOW_CRD_VERSION"]
    # WORKFLOW_INSTANCE_CRD_VERSION = ENV["WORKFLOW_INSTANCE_CRD_VERSION"]

    attr_accessor :kubeclient

    def self.get_kubeclient(opts)
      if @kubeclient = opts[:kubeclient]
        @kubeclient
      else
        kubeclient_version(opts)
      end
    end

    def self.kubeclient_version(opts = {})
      version = opts[:apiVersion] || DEFAULT_API_VERSION

      if existing_version = KubeclientVersions[version]
        return existing_version
      else
        new_instance = new(version).kubeclient
        KubeclientVersions[version] = new_instance
        new_instance
      end
    end
    KubeclientVersions = {}

    # opts can have keys
    #   kubernetes_client - already instantiated kubernetes client
    def initialize(apiVersion)
      ssl_options  = {}
      auth_options = { bearer_token_file: '/var/run/secrets/kubernetes.io/serviceaccount/token' }
      
      if File.exist?("/var/run/secrets/kubernetes.io/serviceaccount/ca.crt")
        ssl_options[:ca_file] = "/var/run/secrets/kubernetes.io/serviceaccount/ca.crt"
      end

      socket_options = {
        socket_class: Celluloid::IO::TCPSocket,
        ssl_socket_class: Celluloid::IO::SSLSocket
      }

      @kubeclient = Kubeclient::Client.new(
        'https://kubernetes.default.svc/apis/',
        "dtk.io/#{apiVersion}",
        auth_options: auth_options,
        ssl_options:  ssl_options,
        socket_options: socket_options 
      )
      @kubeclient.discover unless @kubeclient.discovered
      @kubeclient
    end
  end
end