require 'resolv-replace'
require 'influxdb2/client'
module DTK::State
  class Component::Attribute::Influxdb
    class Client
      INFLUXDB_SECRET_DIR = '/app/influxdb/'
      def initialize
        @connection_parameters   = return_connection_parameters
        @connection    = return_connection(@connection_parameters)
      end

      def query(query_expression)
        query_api = self.connection.create_query_api
        query_api.query(query_expression)
      rescue => e
        raise "Failed while processing flux query!. Error: #{e}"
      end

      def write_point(data)
        write_api = self.connection.create_write_api
        write_api.write(data: data)
      rescue => e
        raise e
      end

      def measurement_helper(measurement_name)
        klass = Measurement.const_get(measurement_name.to_sym.capitalize)
        klass.new(measurement_name, self)
      rescue => e
        raise e
      end

      attr_reader :connection_parameters, :connection
      
      private

      def return_connection_parameters
        params = {}
        Dir.children(INFLUXDB_SECRET_DIR).each do |file| 
          if (file == 'url' || file == 'token' || file == 'org' || file == 'bucket')
            params[file.to_sym] = File.read("#{INFLUXDB_SECRET_DIR}#{file}")
          end
        end
        {
          url: params[:url],
          token: params[:token],
          org: params[:org],
          bucket: params[:bucket]
        }
      rescue => e
        raise "Problem happened while processing InfluxDB connection parameters. Error: #{e}"
      end

      def return_connection(connection_parameters)
        client = InfluxDB2::Client.new(connection_parameters[:url], connection_parameters[:token],
                              bucket: connection_parameters[:bucket],
                              org: connection_parameters[:org],
                              precision: InfluxDB2::WritePrecision::MILLISECOND,
                              use_ssl: false)
        query_api = client.create_query_api
        query_api.query(query: 'from(bucket:"' + connection_parameters[:bucket] + '") |> range(start: -5)')
        client
      rescue => e
        raise "Connection with InfluxDB could not be established. #{e}"
      end
    end
  end
end